package net.krakatoa.proxy.listener;

import com.mongodb.client.model.Filters;
import com.mongodb.client.model.UpdateOptions;
import com.mongodb.client.model.Updates;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import net.krakatoa.proxy.ProxySystem;
import net.md_5.bungee.api.connection.ProxiedPlayer;
import net.md_5.bungee.api.event.PlayerDisconnectEvent;
import net.md_5.bungee.api.plugin.Listener;
import net.md_5.bungee.event.EventHandler;
import org.bson.Document;
import org.bson.conversions.Bson;
import org.reactivestreams.Publisher;
import org.reactivestreams.Subscriber;
import org.reactivestreams.Subscription;
import org.redisson.api.RMap;

public class PlayerDisconnectListener implements Listener {

  @EventHandler
  public void onDisconnect(PlayerDisconnectEvent event) {
    ProxiedPlayer proxiedPlayer = event.getPlayer();

    String uuid = ProxySystem.getInstance().getFormatter()
        .formatUuid(proxiedPlayer.getUniqueId().toString());

    // redis
    RMap<String, String> map = ProxySystem.getInstance().getRedisManager()
        .getRedissonClient()
        .getMap("player:" + uuid);

    // mongo
    Publisher<Document> document = ProxySystem.getInstance().getMongoManager()
        .getPlayersCollection()
        .find(Filters.eq("uuid", uuid)).first();

    document.subscribe(new Subscriber<>() {
      @Override
      public void onSubscribe(Subscription subscription) {
        subscription.request(1);
      }

      @Override
      public void onNext(Document document) {
        List<Bson> updatesList = new ArrayList<>();

        map.forEach((key, value) -> {
          if (!Objects.equals(key, "_id") && !Objects.equals(key, "uuid")) {
            updatesList.add(Updates.set(key, value));
          }
        });

        Bson updates = Updates.combine(updatesList);

        UpdateOptions options = new UpdateOptions().upsert(true);

        System.out.println("doc: " + document.keySet());

        ProxySystem.getInstance().getMongoManager().getPlayersCollection()
            .updateOne(Filters.eq("uuid", uuid), updates);
/*
            ProxySystem.getInstance().getMongoManager().getPlayersCollection()
                .updateOne(document, updates, options, (result, t) -> {
                  System.out.println(result == null);

                  if (result != null) {
                    System.out.println("Modified document count: " + result.getModifiedCount());
                    System.out.println("Upserted id: " + result.getUpsertedId());
                  }
                }); */
      }

      @Override
      public void onError(Throwable throwable) {
        System.out.println("Mongo player is null on disconnect");
      }

      @Override
      public void onComplete() {

      }
    });

    // redis
    if (!map.isEmpty()) {
      map.delete();
    }
  }

}
